pub enum TrySendError<T> {
Full(T),
Disconnected(T),
}Expand description
This enumeration is the list of the possible error outcomes for the try_send method.
Variants
Full(T)
The data could not be sent on the channel because it would require that the callee block to send the data.
If this is a zero-capacity channel, then the error indicates that there was no receiver available to receive the message at the time.
Disconnected(T)
This channel’s receiving half has disconnected, so the data could not be sent. The data is returned back to the callee in this case.
Implementations
Sourceimpl<T> TrySendError<T>
impl<T> TrySendError<T>
Sourcepub fn into_inner(self) -> T
pub fn into_inner(self) -> T
Unwraps the value.
Examples
use crossbeam_channel::bounded;
let (tx, rx) = bounded(0);
if let Err(err) = tx.try_send("foo") {
assert_eq!(err.into_inner(), "foo");
}Trait Implementations
Sourceimpl<T> Clone for TrySendError<T>where
T: Clone,
impl<T> Clone for TrySendError<T>where
T: Clone,
Sourcefn clone(&self) -> TrySendError<T>
fn clone(&self) -> TrySendError<T>
Returns a duplicate of the value. Read more
1.0.0 · Sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
Performs copy-assignment from
source. Read moreSourceimpl<T> Debug for TrySendError<T>
impl<T> Debug for TrySendError<T>
Sourceimpl<T> Display for TrySendError<T>
impl<T> Display for TrySendError<T>
Sourceimpl<T> Error for TrySendError<T>where
T: Send,
impl<T> Error for TrySendError<T>where
T: Send,
Sourcefn description(&self) -> &str
fn description(&self) -> &str
👎Deprecated since 1.42.0: use the Display impl or to_string()
Sourcefn cause(&self) -> Option<&dyn Error>
fn cause(&self) -> Option<&dyn Error>
👎Deprecated since 1.33.0: replaced by Error::source, which can support downcasting
Sourceimpl<T> From<SendError<T>> for TrySendError<T>
impl<T> From<SendError<T>> for TrySendError<T>
Sourcefn from(err: SendError<T>) -> TrySendError<T>
fn from(err: SendError<T>) -> TrySendError<T>
Converts to this type from the input type.
Sourceimpl<T> PartialEq for TrySendError<T>where
T: PartialEq,
impl<T> PartialEq for TrySendError<T>where
T: PartialEq,
impl<T> Copy for TrySendError<T>where
T: Copy,
impl<T> Eq for TrySendError<T>where
T: Eq,
impl<T> StructuralPartialEq for TrySendError<T>