pub enum RecvTimeoutError {
Timeout,
Disconnected,
}Expand description
This enumeration is the list of possible errors that made recv_timeout unable to return
data when called. This can occur with both bounded and unbounded channels.
Variants
Timeout
This channel is currently empty, but the senders have not yet disconnected, so data may yet become available.
Disconnected
The channel’s sending half has become disconnected, and there will never be any more data received on it.