pub struct FileEdit(/* private fields */);Expand description
A read/write file handle with path() attached and improved error messages. Contains methods
and trait implements for both readable and writeable files.
Examples
use std::io::{Read, Seek, Write, SeekFrom};
use path_abs::FileEdit;
let example = "example.txt";
let expected = "foo\nbar";
let mut edit = FileEdit::create(example)?;
let mut s = String::new();
edit.write_all(expected.as_bytes())?;
edit.seek(SeekFrom::Start(0))?;
edit.read_to_string(&mut s)?;
assert_eq!(expected, s);Implementations
Sourceimpl FileEdit
impl FileEdit
Sourcepub fn open<P>(path: P, options: OpenOptions) -> Result<FileEdit, Error>
pub fn open<P>(path: P, options: OpenOptions) -> Result<FileEdit, Error>
Open the file with the given OpenOptions but always sets read and write to true.
Sourcepub fn create<P>(path: P) -> Result<FileEdit, Error>
pub fn create<P>(path: P) -> Result<FileEdit, Error>
Open the file in editing mode, truncating it first if it exists and creating it otherwise.
Sourcepub fn append<P>(path: P) -> Result<FileEdit, Error>
pub fn append<P>(path: P) -> Result<FileEdit, Error>
Open the file for appending, creating it if it doesn’t exist.
Sourcepub fn edit<P>(path: P) -> Result<FileEdit, Error>
pub fn edit<P>(path: P) -> Result<FileEdit, Error>
Open the file for editing (reading and writing) but do not create it if it doesn’t exist.
Sourcepub fn sync_all(&self) -> Result<(), Error>
pub fn sync_all(&self) -> Result<(), Error>
Attempts to sync all OS-internal metadata to disk.
This function will attempt to ensure that all in-core data reaches the filesystem before returning.
This function is identical to std::fs::File::sync_all except it has error messages which include the action and the path.
Sourcepub fn sync_data(&self) -> Result<(), Error>
pub fn sync_data(&self) -> Result<(), Error>
This function is similar to sync_all, except that it may not synchronize file metadata to the filesystem.
This function is identical to std::fs::File::sync_data except it has error messages which include the action and the path.
Sourcepub fn set_len(&mut self, size: u64) -> Result<(), Error>
pub fn set_len(&mut self, size: u64) -> Result<(), Error>
Truncates or extends the underlying file, updating the size of this file to become size.
This function is identical to std::fs::File::set_len except:
- It has error messages which include the action and the path.
- It takes
&mut selfinstead of&self.
Sourcepub fn set_permissions(&mut self, perm: Permissions) -> Result<(), Error>
pub fn set_permissions(&mut self, perm: Permissions) -> Result<(), Error>
Changes the permissions on the underlying file.
This function is identical to std::fs::File::set_permissions except:
- It has error messages which include the action and the path.
- It takes
&mut selfinstead of&self.
Sourcepub fn read_string(&mut self) -> Result<String, Error>
pub fn read_string(&mut self) -> Result<String, Error>
Read what remains of the file to a String.
Methods from Deref<Target = FileOpen>