pub struct LookupError<E> {
pub var_name: String,
pub cause: E,
}Expand description
Represents a variable lookup error.
This error is returned by env_with_context() function (and, therefore, also by env(),
full_with_context() and full()) when the provided context function returns an error. The
original error is provided in the cause field, while name contains the name of a variable
whose expansion caused the error.
Fields
var_name: StringThe name of the problematic variable inside the input string.
cause: EThe original error returned by the context function.