pub struct FileWrite(/* private fields */);Expand description
A write-only file handle with path() attached and improved error messages. Contains only the
methods and trait implementations which are allowed by a write-only file.
Examples
use std::io::Write;
use path_abs::{PathFile, FileWrite};
let example = "example.txt";
let expected = "foo\nbar";
let mut write = FileWrite::create(example)?;
write.write_all(expected.as_bytes())?;
write.flush();
let file = PathFile::new(example)?;
assert_eq!(expected, file.read_string()?);Implementations
Sourceimpl FileWrite
impl FileWrite
Sourcepub fn open<P>(path: P, options: OpenOptions) -> Result<FileWrite, Error>
pub fn open<P>(path: P, options: OpenOptions) -> Result<FileWrite, Error>
Open the file with the given OpenOptions but always sets write to true.
Sourcepub fn create<P>(path: P) -> Result<FileWrite, Error>
pub fn create<P>(path: P) -> Result<FileWrite, Error>
Open the file in write-only mode, truncating it first if it exists and creating it otherwise.
Sourcepub fn append<P>(path: P) -> Result<FileWrite, Error>
pub fn append<P>(path: P) -> Result<FileWrite, Error>
Open the file for appending, creating it if it doesn’t exist.
Sourcepub fn edit<P>(path: P) -> Result<FileWrite, Error>
pub fn edit<P>(path: P) -> Result<FileWrite, Error>
Open the file for editing (reading and writing) but do not create it if it doesn’t exist.
Sourcepub fn sync_all(&self) -> Result<(), Error>
pub fn sync_all(&self) -> Result<(), Error>
Attempts to sync all OS-internal metadata to disk.
This function will attempt to ensure that all in-core data reaches the filesystem before returning.
This function is identical to std::fs::File::sync_all except it has error messages which include the action and the path.
Sourcepub fn sync_data(&self) -> Result<(), Error>
pub fn sync_data(&self) -> Result<(), Error>
This function is similar to sync_all, except that it may not synchronize file metadata to the filesystem.
This function is identical to std::fs::File::sync_data except it has error messages which include the action and the path.
Sourcepub fn set_len(&mut self, size: u64) -> Result<(), Error>
pub fn set_len(&mut self, size: u64) -> Result<(), Error>
Truncates or extends the underlying file, updating the size of this file to become size.
This function is identical to std::fs::File::set_len except:
- It has error messages which include the action and the path.
- It takes
&mut selfinstead of&self.
Sourcepub fn set_permissions(&mut self, perm: Permissions) -> Result<(), Error>
pub fn set_permissions(&mut self, perm: Permissions) -> Result<(), Error>
Changes the permissions on the underlying file.
This function is identical to std::fs::File::set_permissions except:
- It has error messages which include the action and the path.
- It takes
&mut selfinstead of&self.
Methods from Deref<Target = FileOpen>