pub struct RecvError;Expand description
An error returned from the Receiver::recv method.
The [recv] operation can only fail if the sending half of a channel is disconnected and the
channel is empty, implying that no further messages will ever be received.